@@ -1,6 +1,6 @@
 /*
     FUSE: Filesystem in Userspace
-    Copyright (C) 2001  Miklos Szeredi (mszeredi@inf.bme.hu)
+    Copyright (C) 2001-2004  Miklos Szeredi <miklos@szeredi.hu>
 
     This program can be distributed under the terms of the GNU GPL.
     See the file COPYING.
@@ -11,7 +11,7 @@
 #include <linux/init.h>
 #include <linux/slab.h>
 
-MODULE_AUTHOR("Miklos Szeredi <mszeredi@inf.bme.hu>");
+MODULE_AUTHOR("Miklos Szeredi <miklos@szeredi.hu>");
 MODULE_DESCRIPTION("Filesystem in Userspace");
 #ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
@@ -19,14 +19,6 @@ MODULE_LICENSE("GPL");
 
 spinlock_t fuse_lock = SPIN_LOCK_UNLOCKED;
 
-/* Must be called with the fuse lock held */
-void fuse_release_conn(struct fuse_conn *fc)
-{
-	if(fc->sb == NULL && fc->file == NULL) {
-		kfree(fc);
-	}
-}
-
 int __init fuse_init(void)
 {
 	int res;
@@ -36,11 +28,11 @@ int __init fuse_init(void)
 	       FUSE_KERNEL_VERSION, FUSE_KERNEL_MINOR_VERSION);
 
 	res = fuse_fs_init();
-	if(res)
+	if (res)
 		goto err;
 	
 	res = fuse_dev_init();
-	if(res)
+	if (res)
 		goto err_fs_cleanup;
 	
 	return 0;
